@@ -24,7 +24,7 @@ pub fn develop(
 ) -> Result<()> {
     let mut target_triple = cargo_options.target.as_ref().map(|x| x.to_string());
     let target = Target::from_target_triple(cargo_options.target)?;
-    let python = target.get_venv_python(&venv_dir);
+    let python = target.get_venv_python(venv_dir);
 
     // check python platform and architecture
     if !target.user_specified {
@@ -74,7 +74,12 @@ pub fn develop(
 
     // Install dependencies
     if !build_context.metadata21.requires_dist.is_empty() {
-        let mut args = vec!["-m".to_string(), "pip".to_string(), "install".to_string()];
+        let mut args = vec![
+            "-m".to_string(),
+            "pip".to_string(),
+            "install".to_string(),
+            "--disable-pip-version-check".to_string(),
+        ];
         args.extend(build_context.metadata21.requires_dist.iter().map(|x| {
             let mut pkg = x.clone();
             // Remove extra marker to make it installable with pip
@@ -85,7 +90,7 @@ pub fn develop(
             }
             pkg
         }));
-        let status = Command::new(&interpreter.executable)
+        let status = Command::new(interpreter.executable)
             .args(&args)
             .status()
             .context("Failed to run pip install")?;
@@ -105,7 +110,7 @@ pub fn develop(
             "--force-reinstall",
         ];
         let output = Command::new(&python)
-            .args(&command)
+            .args(command)
             .arg(dunce::simplified(filename))
             .output()
             .context(format!("pip install failed with {:?}", python))?;
@@ -121,13 +126,13 @@ pub fn develop(
         }
         if !output.stderr.is_empty() {
             eprintln!(
-                "⚠️  Warning: pip raised a warning running {:?}:\n{}",
+                "⚠️ Warning: pip raised a warning running {:?}:\n{}",
                 &command,
                 String::from_utf8_lossy(&output.stderr).trim(),
             );
         }
         println!(
-            "🛠  Installed {}-{}",
+            "🛠 Installed {}-{}",
             build_context.metadata21.name, build_context.metadata21.version
         );
     }
