@@ -5,8 +5,7 @@
 
 use anyhow::{bail, Context, Result};
 use cargo_zigbuild::Zig;
-use clap::{ArgEnum, IntoApp, Parser, Subcommand};
-use clap_complete::Generator;
+use clap::{CommandFactory, Parser, Subcommand};
 use maturin::{
     develop, init_project, new_project, write_dist_info, BridgeModel, BuildOptions, CargoOptions,
     GenerateProjectOptions, PathWriter, PlatformTag, PythonInterpreter, Target,
@@ -16,83 +15,86 @@ use maturin::{upload_ui, PublishOpt};
 use std::env;
 use std::io;
 use std::path::PathBuf;
-use std::str::FromStr;
 
 #[derive(Debug, Parser)]
-#[clap(
+#[command(
     version,
     name = env!("CARGO_PKG_NAME"),
-    global_setting(clap::AppSettings::DeriveDisplayOrder)
+    display_order = 1,
 )]
 #[cfg_attr(feature = "cargo-clippy", allow(clippy::large_enum_variant))]
 /// Build and publish crates with pyo3, rust-cpython and cffi bindings as well
 /// as rust binaries as python packages
 enum Opt {
-    #[clap(name = "build", alias = "b")]
+    #[command(name = "build", alias = "b")]
     /// Build the crate into python packages
     Build {
         /// Build artifacts in release mode, with optimizations
-        #[clap(short = 'r', long)]
+        #[arg(short = 'r', long)]
         release: bool,
         /// Strip the library for minimum file size
-        #[clap(long)]
+        #[arg(long)]
         strip: bool,
         /// Build a source distribution
-        #[clap(long)]
+        #[arg(long)]
         sdist: bool,
-        #[clap(flatten)]
+        #[command(flatten)]
         build: BuildOptions,
     },
     #[cfg(feature = "upload")]
-    #[clap(name = "publish")]
+    #[command(name = "publish")]
     /// Build and publish the crate as python packages to pypi
     Publish {
         /// Do not pass --release to cargo
-        #[clap(long)]
+        #[arg(long)]
         debug: bool,
         /// Do not strip the library for minimum file size
-        #[clap(long = "no-strip")]
+        #[arg(long = "no-strip")]
         no_strip: bool,
         /// Don't build a source distribution
-        #[clap(long = "no-sdist")]
+        #[arg(long = "no-sdist")]
         no_sdist: bool,
-        #[clap(flatten)]
+        #[command(flatten)]
         publish: PublishOpt,
-        #[clap(flatten)]
+        #[command(flatten)]
         build: BuildOptions,
     },
-    #[clap(name = "list-python")]
+    #[command(name = "list-python")]
     /// Search and list the available python installations
     ListPython {
-        #[clap(long)]
+        #[arg(long)]
         target: Option<String>,
     },
-    #[clap(name = "develop", alias = "dev")]
+    #[command(name = "develop", alias = "dev")]
     /// Install the crate as module in the current virtualenv
     ///
     /// Note that this command doesn't create entrypoints
     Develop {
-        /// Which kind of bindings to use. Possible values are pyo3, rust-cpython, cffi and bin
-        #[clap(short = 'b', long = "bindings", alias = "binding-crate")]
+        /// Which kind of bindings to use
+        #[arg(
+            short = 'b',
+            long = "bindings",
+            alias = "binding-crate",
+            value_parser = ["pyo3", "pyo3-ffi", "rust-cpython", "cffi", "uniffi", "bin"]
+        )]
         bindings: Option<String>,
         /// Pass --release to cargo
-        #[clap(short = 'r', long)]
+        #[arg(short = 'r', long)]
         release: bool,
         /// Strip the library for minimum file size
-        #[clap(long)]
+        #[arg(long)]
         strip: bool,
         /// Install extra requires aka. optional dependencies
         ///
         /// Use as `--extras=extra1,extra2`
-        #[clap(
+        #[arg(
             short = 'E',
             long,
-            use_value_delimiter = true,
-            multiple_values = false,
-            multiple_occurrences = false
+            value_delimiter = ',',
+            action = clap::ArgAction::Append
         )]
         extras: Vec<String>,
-        #[clap(flatten)]
+        #[command(flatten)]
         cargo_options: CargoOptions,
     },
     /// Build only a source distribution (sdist) without compiling.
@@ -100,30 +102,30 @@ enum Opt {
     /// Building a source distribution requires a pyproject.toml with a `[build-system]` table.
     ///
     /// This command is a workaround for [pypa/pip#6041](https://github.com/pypa/pip/issues/6041)
-    #[clap(name = "sdist")]
+    #[command(name = "sdist")]
     SDist {
-        #[clap(short = 'm', long = "manifest-path", parse(from_os_str))]
+        #[arg(short = 'm', long = "manifest-path")]
         /// The path to the Cargo.toml
         manifest_path: Option<PathBuf>,
         /// The directory to store the built wheels in. Defaults to a new "wheels"
         /// directory in the project's target directory
-        #[clap(short, long, parse(from_os_str))]
+        #[arg(short, long)]
         out: Option<PathBuf>,
     },
     /// Create a new cargo project in an existing directory
-    #[clap(name = "init")]
+    #[command(name = "init")]
     InitProject {
         /// Project path
         path: Option<String>,
-        #[clap(flatten)]
+        #[command(flatten)]
         options: GenerateProjectOptions,
     },
     /// Create a new cargo project
-    #[clap(name = "new")]
+    #[command(name = "new")]
     NewProject {
         /// Project path
         path: String,
-        #[clap(flatten)]
+        #[command(flatten)]
         options: GenerateProjectOptions,
     },
     /// Upload python packages to pypi
@@ -131,27 +133,27 @@ enum Opt {
     /// It is mostly similar to `twine upload`, but can only upload python wheels
     /// and source distributions.
     #[cfg(feature = "upload")]
-    #[clap(name = "upload")]
+    #[command(name = "upload")]
     Upload {
-        #[clap(flatten)]
+        #[command(flatten)]
         publish: PublishOpt,
         /// The python packages to upload
-        #[clap(name = "FILE", parse(from_os_str))]
+        #[arg(value_name = "FILE")]
         files: Vec<PathBuf>,
     },
     /// Backend for the PEP 517 integration. Not for human consumption
     ///
     /// The commands are meant to be called from the python PEP 517
-    #[clap(subcommand)]
+    #[command(subcommand)]
     Pep517(Pep517Command),
     /// Generate shell completions
-    #[clap(name = "completions", hide = true)]
+    #[command(name = "completions", hide = true)]
     Completions {
-        #[clap(name = "SHELL", parse(try_from_str))]
-        shell: Shell,
+        #[arg(value_name = "SHELL")]
+        shell: clap_complete_command::Shell,
     },
     /// Zig linker wrapper
-    #[clap(subcommand, hide = true)]
+    #[command(subcommand, hide = true)]
     Zig(Zig),
 }
 
@@ -159,79 +161,46 @@ enum Opt {
 ///
 /// The commands are meant to be called from the python PEP 517
 #[derive(Debug, Subcommand)]
-#[clap(name = "pep517", hide = true)]
+#[command(name = "pep517", hide = true)]
 enum Pep517Command {
     /// The implementation of prepare_metadata_for_build_wheel
-    #[clap(name = "write-dist-info")]
+    #[command(name = "write-dist-info")]
     WriteDistInfo {
-        #[clap(flatten)]
+        #[command(flatten)]
         build_options: BuildOptions,
         /// The metadata_directory argument to prepare_metadata_for_build_wheel
-        #[clap(long = "metadata-directory", parse(from_os_str))]
+        #[arg(long = "metadata-directory")]
         metadata_directory: PathBuf,
         /// Strip the library for minimum file size
-        #[clap(long)]
+        #[arg(long)]
         strip: bool,
     },
-    #[clap(name = "build-wheel")]
+    #[command(name = "build-wheel")]
     /// Implementation of build_wheel
     ///
     /// --release and --strip are currently unused by the PEP 517 implementation
     BuildWheel {
-        #[clap(flatten)]
+        #[command(flatten)]
         build_options: BuildOptions,
         /// Strip the library for minimum file size
-        #[clap(long)]
+        #[arg(long)]
         strip: bool,
         /// Build editable wheels
-        #[clap(long)]
+        #[arg(long)]
         editable: bool,
     },
     /// The implementation of build_sdist
-    #[clap(name = "write-sdist")]
+    #[command(name = "write-sdist")]
     WriteSDist {
         /// The sdist_directory argument to build_sdist
-        #[clap(long = "sdist-directory", parse(from_os_str))]
+        #[arg(long = "sdist-directory")]
         sdist_directory: PathBuf,
-        #[clap(
-            short = 'm',
-            long = "manifest-path",
-            parse(from_os_str),
-            default_value = "Cargo.toml",
-            name = "PATH"
-        )]
+        #[arg(short = 'm', long = "manifest-path", value_name = "PATH")]
         /// The path to the Cargo.toml
-        manifest_path: PathBuf,
+        manifest_path: Option<PathBuf>,
     },
 }
 
-#[derive(Debug, Clone, Copy, ArgEnum)]
-#[allow(clippy::enum_variant_names)]
-enum Shell {
-    Bash,
-    Elvish,
-    Fish,
-    PowerShell,
-    Zsh,
-    Fig,
-}
-
-impl FromStr for Shell {
-    type Err = String;
-
-    fn from_str(s: &str) -> Result<Self, Self::Err> {
-        match s.to_ascii_lowercase().as_str() {
-            "bash" => Ok(Shell::Bash),
-            "elvish" => Ok(Shell::Elvish),
-            "fish" => Ok(Shell::Fish),
-            "powershell" => Ok(Shell::PowerShell),
-            "zsh" => Ok(Shell::Zsh),
-            "fig" => Ok(Shell::Fig),
-            _ => Err("[valid values: bash, elvish, fish, powershell, zsh, fig]".to_string()),
-        }
-    }
-}
-
 /// Dispatches into the native implementations of the PEP 517 functions
 ///
 /// The last line of stdout is used as return value from the python part of the implementation
@@ -260,7 +229,7 @@ fn pep517(subcommand: Pep517Command) -> Result<()> {
                         .get_platform_tag(&[PlatformTag::Linux], context.universal2)?;
                     vec![format!("cp{}{}-abi3-{}", major, minor, platform)]
                 }
-                BridgeModel::Bin(None) | BridgeModel::Cffi => {
+                BridgeModel::Bin(None) | BridgeModel::Cffi | BridgeModel::UniFfi => {
                     context
                         .target
                         .get_universal_tags(&[PlatformTag::Linux], context.universal2)?
@@ -289,7 +258,7 @@ fn pep517(subcommand: Pep517Command) -> Result<()> {
             let build_options = BuildOptions {
                 out: Some(sdist_directory),
                 cargo: CargoOptions {
-                    manifest_path: Some(manifest_path),
+                    manifest_path,
                     ..Default::default()
                 },
                 ..Default::default()
@@ -424,37 +393,14 @@ fn run() -> Result<()> {
         #[cfg(feature = "upload")]
         Opt::Upload { publish, files } => {
             if files.is_empty() {
-                println!("⚠️  Warning: No files given, exiting.");
+                eprintln!("⚠️  Warning: No files given, exiting.");
                 return Ok(());
             }
 
             upload_ui(&files, &publish)?
         }
         Opt::Completions { shell } => {
-            let mut cmd = Opt::command();
-            match shell {
-                Shell::Fig => {
-                    cmd.set_bin_name(env!("CARGO_BIN_NAME"));
-                    let fig = clap_complete_fig::Fig;
-                    fig.generate(&cmd, &mut io::stdout());
-                }
-                _ => {
-                    let shell = match shell {
-                        Shell::Bash => clap_complete::Shell::Bash,
-                        Shell::Elvish => clap_complete::Shell::Elvish,
-                        Shell::Fish => clap_complete::Shell::Fish,
-                        Shell::PowerShell => clap_complete::Shell::PowerShell,
-                        Shell::Zsh => clap_complete::Shell::Zsh,
-                        Shell::Fig => unreachable!(),
-                    };
-                    clap_complete::generate(
-                        shell,
-                        &mut cmd,
-                        env!("CARGO_BIN_NAME"),
-                        &mut io::stdout(),
-                    )
-                }
-            }
+            shell.generate(&mut Opt::command(), &mut io::stdout());
         }
         Opt::Zig(subcommand) => {
             subcommand
@@ -467,11 +413,6 @@ fn run() -> Result<()> {
 }
 
 fn main() {
-    #[cfg(feature = "human-panic")]
-    {
-        human_panic::setup_panic!();
-    }
-
     if let Err(e) = run() {
         eprintln!("💥 maturin failed");
         for cause in e.chain().collect::<Vec<_>>().iter() {
