@@ -10,7 +10,8 @@ use std::fmt;
 use std::path::Path;
 use std::path::PathBuf;
 use std::str;
-use target_lexicon::{Environment, Triple};
+use target_lexicon::{Architecture, Environment, Triple};
+use tracing::error;
 
 pub(crate) const RUST_1_64_0: semver::Version = semver::Version::new(1, 64, 0);
 
@@ -25,9 +26,11 @@ pub enum Os {
     NetBsd,
     OpenBsd,
     Dragonfly,
+    Solaris,
     Illumos,
     Haiku,
     Emscripten,
+    Wasi,
 }
 
 impl fmt::Display for Os {
@@ -35,14 +38,16 @@ impl fmt::Display for Os {
         match *self {
             Os::Linux => write!(f, "Linux"),
             Os::Windows => write!(f, "Windows"),
-            Os::Macos => write!(f, "MacOS"),
+            Os::Macos => write!(f, "macOS"),
             Os::FreeBsd => write!(f, "FreeBSD"),
             Os::NetBsd => write!(f, "NetBSD"),
             Os::OpenBsd => write!(f, "OpenBSD"),
             Os::Dragonfly => write!(f, "DragonFly"),
+            Os::Solaris => write!(f, "Solaris"),
             Os::Illumos => write!(f, "Illumos"),
             Os::Haiku => write!(f, "Haiku"),
             Os::Emscripten => write!(f, "Emscripten"),
+            Os::Wasi => write!(f, "Wasi"),
         }
     }
 }
@@ -92,6 +97,29 @@ impl fmt::Display for Arch {
     }
 }
 
+impl Arch {
+    /// Represents the hardware platform.
+    ///
+    /// This is the same as the native platform's `uname -m` output.
+    pub fn machine(&self) -> &'static str {
+        // See https://www.freebsd.org/cgi/man.cgi?query=arch&sektion=7&format=html
+        // MACHINE_ARCH	vs MACHINE_CPUARCH vs MACHINE section
+        match self {
+            Arch::Aarch64 => "arm64",
+            Arch::Armv6L | Arch::Armv7L => "arm",
+            Arch::Powerpc | Arch::Powerpc64Le | Arch::Powerpc64 => "powerpc",
+            Arch::X86 => "i386",
+            Arch::X86_64 => "amd64",
+            Arch::Riscv64 => "riscv",
+            Arch::Mips64el | Arch::Mipsel => "mips",
+            // sparc64 is unsupported since FreeBSD 13.0
+            Arch::Sparc64 => "sparc64",
+            Arch::Wasm32 => "wasm32",
+            Arch::S390X => "s390x",
+        }
+    }
+}
+
 // Returns the set of supported architectures for each operating system
 fn get_supported_architectures(os: &Os) -> Vec<Arch> {
     match os {
@@ -112,18 +140,36 @@ fn get_supported_architectures(os: &Os) -> Vec<Arch> {
         ],
         Os::Windows => vec![Arch::X86, Arch::X86_64, Arch::Aarch64],
         Os::Macos => vec![Arch::Aarch64, Arch::X86_64],
-        Os::NetBsd => vec![Arch::Aarch64, Arch::X86, Arch::X86_64],
-        Os::FreeBsd => vec![
+        Os::FreeBsd | Os::NetBsd => vec![
             Arch::Aarch64,
+            Arch::Armv6L,
+            Arch::Armv7L,
+            Arch::Powerpc,
             Arch::Powerpc64,
             Arch::Powerpc64Le,
+            Arch::X86,
             Arch::X86_64,
+            Arch::Riscv64,
+            Arch::Mips64el,
+            Arch::Mipsel,
+            Arch::Sparc64,
+        ],
+        Os::OpenBsd => vec![
+            Arch::X86,
+            Arch::X86_64,
+            Arch::Aarch64,
+            Arch::Armv7L,
+            Arch::Powerpc,
+            Arch::Powerpc64,
+            Arch::Powerpc64Le,
+            Arch::Riscv64,
+            Arch::Sparc64,
         ],
-        Os::OpenBsd => vec![Arch::X86, Arch::X86_64, Arch::Aarch64],
         Os::Dragonfly => vec![Arch::X86_64],
         Os::Illumos => vec![Arch::X86_64],
         Os::Haiku => vec![Arch::X86_64],
-        Os::Emscripten => vec![Arch::Wasm32],
+        Os::Solaris => vec![Arch::X86_64, Arch::Sparc64],
+        Os::Emscripten | Os::Wasi => vec![Arch::Wasm32],
     }
 }
 
@@ -148,7 +194,7 @@ impl Target {
     /// Fails if the target triple isn't supported
     pub fn from_target_triple(target_triple: Option<String>) -> Result<Self> {
         use target_lexicon::{
-            Architecture, ArmArchitecture, Mips32Architecture, Mips64Architecture, OperatingSystem,
+            ArmArchitecture, Mips32Architecture, Mips64Architecture, OperatingSystem,
         };
 
         let rustc_version = rustc_version_meta()?;
@@ -173,9 +219,11 @@ impl Target {
             OperatingSystem::Freebsd => Os::FreeBsd,
             OperatingSystem::Openbsd => Os::OpenBsd,
             OperatingSystem::Dragonfly => Os::Dragonfly,
+            OperatingSystem::Solaris => Os::Solaris,
             OperatingSystem::Illumos => Os::Illumos,
             OperatingSystem::Haiku => Os::Haiku,
             OperatingSystem::Emscripten => Os::Emscripten,
+            OperatingSystem::Wasi => Os::Wasi,
             unsupported => bail!("The operating system {:?} is not supported", unsupported),
         };
 
@@ -223,35 +271,53 @@ impl Target {
         universal2: bool,
     ) -> Result<String> {
         let tag = match (&self.os, &self.arch) {
+            // Windows
+            (Os::Windows, Arch::X86) => "win32".to_string(),
+            (Os::Windows, Arch::X86_64) => "win_amd64".to_string(),
+            (Os::Windows, Arch::Aarch64) => "win_arm64".to_string(),
+            // Linux
+            (Os::Linux, _) => {
+                let arch = self.get_platform_arch()?;
+                let mut platform_tags = platform_tags.to_vec();
+                platform_tags.sort();
+                let mut tags = vec![];
+                for platform_tag in platform_tags {
+                    tags.push(format!("{}_{}", platform_tag, arch));
+                    for alias in platform_tag.aliases() {
+                        tags.push(format!("{}_{}", alias, arch));
+                    }
+                }
+                tags.join(".")
+            }
+            // macOS
+            (Os::Macos, Arch::X86_64) | (Os::Macos, Arch::Aarch64) => {
+                let ((x86_64_major, x86_64_minor), (arm64_major, arm64_minor)) = macosx_deployment_target(env::var("MACOSX_DEPLOYMENT_TARGET").ok().as_deref(), universal2)?;
+                if universal2 {
+                    format!(
+                        "macosx_{x86_64_major}_{x86_64_minor}_x86_64.macosx_{arm64_major}_{arm64_minor}_arm64.macosx_{x86_64_major}_{x86_64_minor}_universal2",
+                        x86_64_major = x86_64_major,
+                        x86_64_minor = x86_64_minor,
+                        arm64_major = arm64_major,
+                        arm64_minor = arm64_minor
+                    )
+                } else if self.arch == Arch::Aarch64 {
+                    format!("macosx_{}_{}_arm64", arm64_major, arm64_minor)
+                } else {
+                    format!("macosx_{}_{}_x86_64", x86_64_major, x86_64_minor)
+                }
+            }
             // FreeBSD
-            (Os::FreeBsd, Arch::X86_64)
-            | (Os::FreeBsd, Arch::Aarch64)
-            | (Os::FreeBsd, Arch::Powerpc64)
-            | (Os::FreeBsd, Arch::Powerpc64Le)
+            (Os::FreeBsd, _)
             // NetBSD
-            | (Os::NetBsd, Arch::X86)
-            | (Os::NetBsd, Arch::X86_64)
-            | (Os::NetBsd, Arch::Aarch64)
+            | (Os::NetBsd, _)
             // OpenBSD
-            | (Os::OpenBsd, Arch::X86)
-            | (Os::OpenBsd, Arch::X86_64)
-            | (Os::OpenBsd, Arch::Aarch64) => {
+            | (Os::OpenBsd, _) => {
                 let release = self.get_platform_release()?;
-                let arch = match self.arch {
-                    Arch::X86_64 => "amd64",
-                    Arch::X86 => "i386",
-                    Arch::Aarch64 => "arm64",
-                    Arch::Powerpc64 => "powerpc64",
-                    Arch::Powerpc64Le => "powerpc64le",
-                    _ => panic!(
-                        "unsupported architecture should not have reached get_platform_tag()"
-                    ),
-                };
                 format!(
                     "{}_{}_{}",
                     self.os.to_string().to_ascii_lowercase(),
                     release,
-                    arch
+                    self.arch.machine(),
                 )
             }
             // DragonFly
@@ -266,83 +332,6 @@ impl Target {
                     "x86_64"
                 )
             }
-            // Illumos
-            (Os::Illumos, Arch::X86_64) => {
-                let info = PlatformInfo::new()?;
-                let mut release = info.release().replace('.', "_").replace('-', "_");
-                let mut arch = info.machine().replace(' ', "_").replace('/', "_");
-
-                let mut os = self.os.to_string().to_ascii_lowercase();
-                // See https://github.com/python/cpython/blob/46c8d915715aa2bd4d697482aa051fe974d440e1/Lib/sysconfig.py#L722-L730
-                if let Some((major, other)) = release.split_once('_') {
-                    let major_ver: u64 = major.parse().context("illumos major version is not a number")?;
-                    if major_ver >= 5 {
-                        // SunOS 5 == Solaris 2
-                        os = "solaris".to_string();
-                        release = format!("{}_{}", major_ver - 3, other);
-                        arch = format!("{}_64bit", arch);
-                    }
-                }
-                format!(
-                    "{}_{}_{}",
-                    os,
-                    release,
-                    arch
-                )
-            }
-            // Linux
-            (Os::Linux, _) => {
-                let arch = if self.cross_compiling {
-                    self.arch.to_string()
-                } else {
-                    PlatformInfo::new()
-                        .map(|info| info.machine().into_owned())
-                        .unwrap_or_else(|_| self.arch.to_string())
-                };
-                let mut platform_tags = platform_tags.to_vec();
-                platform_tags.sort();
-                let mut tags = vec![];
-                for platform_tag in platform_tags {
-                    tags.push(format!("{}_{}", platform_tag, arch));
-                    for alias in platform_tag.aliases() {
-                        tags.push(format!("{}_{}", alias, arch));
-                    }
-                }
-                tags.join(".")
-            }
-            // macOS
-            (Os::Macos, Arch::X86_64) => {
-                let ((x86_64_major, x86_64_minor), (arm64_major, arm64_minor)) = macosx_deployment_target(env::var("MACOSX_DEPLOYMENT_TARGET").ok().as_deref(), universal2)?;
-                if universal2 {
-                    format!(
-                        "macosx_{x86_64_major}_{x86_64_minor}_x86_64.macosx_{arm64_major}_{arm64_minor}_arm64.macosx_{x86_64_major}_{x86_64_minor}_universal2",
-                        x86_64_major = x86_64_major,
-                        x86_64_minor = x86_64_minor,
-                        arm64_major = arm64_major,
-                        arm64_minor = arm64_minor
-                    )
-                } else {
-                    format!("macosx_{}_{}_x86_64", x86_64_major, x86_64_minor)
-                }
-            }
-            (Os::Macos, Arch::Aarch64) => {
-                let ((x86_64_major, x86_64_minor), (arm64_major, arm64_minor)) = macosx_deployment_target(env::var("MACOSX_DEPLOYMENT_TARGET").ok().as_deref(), universal2)?;
-                if universal2 {
-                    format!(
-                        "macosx_{x86_64_major}_{x86_64_minor}_x86_64.macosx_{arm64_major}_{arm64_minor}_arm64.macosx_{x86_64_major}_{x86_64_minor}_universal2",
-                        x86_64_major = x86_64_major,
-                        x86_64_minor = x86_64_minor,
-                        arm64_major = arm64_major,
-                        arm64_minor = arm64_minor
-                    )
-                } else {
-                    format!("macosx_{}_{}_arm64", arm64_major, arm64_minor)
-                }
-            }
-            // Windows
-            (Os::Windows, Arch::X86) => "win32".to_string(),
-            (Os::Windows, Arch::X86_64) => "win_amd64".to_string(),
-            (Os::Windows, Arch::Aarch64) => "win_arm64".to_string(),
             // Emscripten
             (Os::Emscripten, Arch::Wasm32) => {
                 let os_version = env::var("MATURIN_EMSCRIPTEN_VERSION");
@@ -350,14 +339,68 @@ impl Target {
                     Ok(os_ver) => os_ver,
                     Err(_) => emcc_version()?,
                 };
-                let release = release.replace('.', "_").replace('-', "_");
+                let release = release.replace(['.', '-'], "_");
                 format!("emscripten_{}_wasm32", release)
             }
-            (_, _) => panic!("unsupported target should not have reached get_platform_tag()"),
+            (Os::Wasi, Arch::Wasm32) => {
+                "any".to_string()
+            }
+            // osname_release_machine fallback for any POSIX system
+            (_, _) => {
+                let info = PlatformInfo::new()?;
+                let mut release = info.release().replace(['.', '-'], "_");
+                let mut machine = info.machine().replace([' ', '/'], "_");
+
+                let mut os = self.os.to_string().to_ascii_lowercase();
+                // See https://github.com/python/cpython/blob/46c8d915715aa2bd4d697482aa051fe974d440e1/Lib/sysconfig.py#L722-L730
+                if os.starts_with("sunos") {
+                    // Solaris / Illumos
+                    if let Some((major, other)) = release.split_once('_') {
+                        let major_ver: u64 = major.parse().context("illumos major version is not a number")?;
+                        if major_ver >= 5 {
+                            // SunOS 5 == Solaris 2
+                            os = "solaris".to_string();
+                            release = format!("{}_{}", major_ver - 3, other);
+                            machine = format!("{}_64bit", machine);
+                        }
+                    }
+                }
+                format!(
+                    "{}_{}_{}",
+                    os,
+                    release,
+                    machine
+                )
+            }
         };
         Ok(tag)
     }
 
+    fn get_platform_arch(&self) -> Result<String> {
+        if self.cross_compiling {
+            return Ok(self.arch.to_string());
+        }
+        let machine = PlatformInfo::new().map(|info| info.machine().into_owned());
+        let arch = match machine {
+            Ok(machine) => {
+                let linux32 = (machine == "x86_64" && self.arch != Arch::X86_64)
+                    || (machine == "aarch64" && self.arch != Arch::Aarch64);
+                if linux32 {
+                    // When running in Docker sometimes uname returns 64-bit architecture while the container is actually 32-bit,
+                    // In this case we trust the architecture of rustc target
+                    self.arch.to_string()
+                } else {
+                    machine
+                }
+            }
+            Err(err) => {
+                error!("Failed to get machine architecture: {}", err);
+                self.arch.to_string()
+            }
+        };
+        Ok(arch)
+    }
+
     fn get_platform_release(&self) -> Result<String> {
         let os = self.os.to_string();
         let os_version = env::var(format!("MATURIN_{}_VERSION", os.to_ascii_uppercase()));
@@ -368,7 +411,7 @@ impl Target {
                 info.release().to_string()
             }
         };
-        let release = release.replace('.', "_").replace('-', "_");
+        let release = release.replace(['.', '-'], "_");
         Ok(release)
     }
 
@@ -403,9 +446,12 @@ impl Target {
             Os::NetBsd => "netbsd",
             Os::OpenBsd => "openbsd",
             Os::Dragonfly => "dragonfly",
+            Os::Solaris => "sunos",
             Os::Illumos => "sunos",
             Os::Haiku => "haiku",
             Os::Emscripten => "emscripten",
+            // This isn't real, there's no sys.platform here
+            Os::Wasi => "wasi",
         }
     }
 
@@ -474,9 +520,11 @@ impl Target {
             | Os::NetBsd
             | Os::OpenBsd
             | Os::Dragonfly
+            | Os::Solaris
             | Os::Illumos
             | Os::Haiku
-            | Os::Emscripten => true,
+            | Os::Emscripten
+            | Os::Wasi => true,
         }
     }
 
@@ -535,6 +583,11 @@ impl Target {
         self.os == Os::Emscripten
     }
 
+    /// Returns true if we're building a binary for wasm32-wasi
+    pub fn is_wasi(&self) -> bool {
+        self.os == Os::Wasi
+    }
+
     /// Returns true if the current platform's target env is Musl
     pub fn is_musl_target(&self) -> bool {
         matches!(
@@ -664,21 +717,27 @@ fn rustc_version_meta() -> Result<VersionMeta> {
     Ok(meta)
 }
 
+/// Get the default macOS deployment target version
 fn macosx_deployment_target(
     deploy_target: Option<&str>,
     universal2: bool,
-) -> Result<((usize, usize), (usize, usize))> {
-    let x86_64_default = if universal2 { (10, 9) } else { (10, 7) };
-    let arm64_default = (11, 0);
+) -> Result<((u16, u16), (u16, u16))> {
+    let x86_64_default_rustc = rustc_macosx_target_version("x86_64-apple-darwin");
+    let x86_64_default = if universal2 && x86_64_default_rustc.1 < 9 {
+        (10, 9)
+    } else {
+        x86_64_default_rustc
+    };
+    let arm64_default = rustc_macosx_target_version("aarch64-apple-darwin");
     let mut x86_64_ver = x86_64_default;
     let mut arm64_ver = arm64_default;
     if let Some(deploy_target) = deploy_target {
         let err_ctx = "MACOSX_DEPLOYMENT_TARGET is invalid";
         let mut parts = deploy_target.split('.');
         let major = parts.next().context(err_ctx)?;
-        let major: usize = major.parse().context(err_ctx)?;
+        let major: u16 = major.parse().context(err_ctx)?;
         let minor = parts.next().context(err_ctx)?;
-        let minor: usize = minor.parse().context(err_ctx)?;
+        let minor: u16 = minor.parse().context(err_ctx)?;
         if (major, minor) > x86_64_default {
             x86_64_ver = (major, minor);
         }
@@ -689,6 +748,47 @@ fn macosx_deployment_target(
     Ok((x86_64_ver, arm64_ver))
 }
 
+pub(crate) fn rustc_macosx_target_version(target: &str) -> (u16, u16) {
+    use std::process::Command;
+    use target_lexicon::OperatingSystem;
+
+    let fallback_version = if target == "aarch64-apple-darwin" {
+        (11, 0)
+    } else {
+        (10, 7)
+    };
+
+    let rustc_target_version = || -> Result<(u16, u16)> {
+        let cmd = Command::new("rustc")
+            .arg("-Z")
+            .arg("unstable-options")
+            .arg("--print")
+            .arg("target-spec-json")
+            .arg("--target")
+            .arg(target)
+            .env("RUSTC_BOOTSTRAP", "1")
+            .output()
+            .context("Failed to run rustc to get the target spec")?;
+        let stdout = String::from_utf8(cmd.stdout).context("rustc output is not valid utf-8")?;
+        let spec: serde_json::Value =
+            serde_json::from_str(&stdout).context("rustc output is not valid json")?;
+        let llvm_target = spec
+            .as_object()
+            .context("rustc output is not a json object")?
+            .get("llvm-target")
+            .context("rustc output does not contain llvm-target")?
+            .as_str()
+            .context("llvm-target is not a string")?;
+        let triple: Triple = llvm_target.parse()?;
+        let (major, minor) = match triple.operating_system {
+            OperatingSystem::MacOSX { major, minor, .. } => (major, minor),
+            _ => fallback_version,
+        };
+        Ok((major, minor))
+    };
+    rustc_target_version().unwrap_or(fallback_version)
+}
+
 fn emcc_version() -> Result<String> {
     use regex::bytes::Regex;
     use std::process::Command;
